@@ -8,12 +8,9 @@ import os
 import re
 import importlib
 
-from azure.cli.core.commands.arm import ArmTemplateBuilder
+from urllib.parse import urlparse
 
-try:
-    from urllib.parse import urlparse
-except ImportError:
-    from urlparse import urlparse  # pylint: disable=import-error
+from azure.cli.core.commands.arm import ArmTemplateBuilder
 
 from knack.log import get_logger
 from knack.util import CLIError
@@ -76,11 +73,11 @@ def log_pprint_template(template):
 
 
 def check_existence(cli_ctx, value, resource_group, provider_namespace, resource_type,
-                    parent_name=None, parent_type=None):
+                    parent_name=None, parent_type=None, static_version=None):
     # check for name or ID and set the type flags
     from azure.cli.core.commands.client_factory import get_mgmt_service_client
     from azure.core.exceptions import HttpResponseError
-    from msrestazure.tools import parse_resource_id
+    from azure.mgmt.core.tools import parse_resource_id
     from azure.cli.core.profiles import ResourceType
     id_parts = parse_resource_id(value)
     resource_client = get_mgmt_service_client(cli_ctx, ResourceType.MGMT_RESOURCE_RESOURCES,
@@ -96,7 +93,10 @@ def check_existence(cli_ctx, value, resource_group, provider_namespace, resource
         parent_path = ''
         resource_name = id_parts['name']
         resource_type = id_parts.get('type', resource_type)
+
     api_version = _resolve_api_version(cli_ctx, provider_namespace, resource_type, parent_path)
+    if static_version:  # only for vnet
+        api_version = static_version
 
     try:
         resource_client.get(rg, ns, parent_path, resource_type, resource_name, api_version)
@@ -121,19 +121,19 @@ def get_key_vault_base_url(cli_ctx, vault_name):
 
 
 def list_sku_info(cli_ctx, location=None):
-    from ._client_factory import _compute_client_factory
+    from .aaz.latest.vm import ListSkus as _ListSkus
 
     def _match_location(loc, locations):
-        return next((x for x in locations if x.lower() == loc.lower()), None)
+        return next((x for x in locations if str(x).lower() == str(loc).lower()), None)
 
-    client = _compute_client_factory(cli_ctx)
-    result = client.resource_skus.list()
+    result = _ListSkus(cli_ctx=cli_ctx)(command_args={})
     if location:
-        result = [r for r in result if _match_location(location, r.locations)]
+        result = [r for r in result if _match_location(location, r['locations'])]
     return result
 
 
-def is_sku_available(cmd, sku_info, zone):
+# pylint: disable=line-too-long
+def is_sku_available(sku_info, zone):
     """
     The SKU is unavailable in the following cases:
     1. regional restriction and the region is restricted
@@ -143,19 +143,15 @@ def is_sku_available(cmd, sku_info, zone):
     is_available = True
     is_restrict_zone = False
     is_restrict_location = False
-    if not sku_info.restrictions:
+    if not sku_info.get('restrictions', []):
         return is_available
-    for restriction in sku_info.restrictions:
-        if restriction.reason_code == 'NotAvailableForSubscription':
-            # The attribute location_info is not supported in versions 2017-03-30 and earlier
-            if cmd.supported_api_version(max_api='2017-03-30'):
-                is_available = False
-                break
-            if restriction.type == 'Zone' and not (
-                    set(sku_info.location_info[0].zones or []) - set(restriction.restriction_info.zones or [])):
+    for restriction in sku_info['restrictions']:
+        if restriction.get('reasonCode', '') == 'NotAvailableForSubscription':
+            if restriction['type'] == 'Zone' and not (
+                    set(sku_info['locationInfo'][0].get('zones', []) or []) - set(restriction['restrictionInfo'].get('zones', []) or [])):
                 is_restrict_zone = True
-            if restriction.type == 'Location' and (
-                    sku_info.location_info[0].location in (restriction.restriction_info.locations or [])):
+            if restriction['type'] == 'Location' and (
+                    sku_info['locationInfo'][0]['location'] in (restriction['restrictionInfo'].get('locations', []) or [])):
                 is_restrict_location = True
 
             if is_restrict_location or (is_restrict_zone and zone):
@@ -172,7 +168,7 @@ def normalize_disk_info(image_data_disks=None,
                         data_disk_delete_option=None, source_snapshots_or_disks=None,
                         source_snapshots_or_disks_size_gb=None, source_disk_restore_point=None,
                         source_disk_restore_point_size_gb=None):
-    from msrestazure.tools import is_valid_resource_id
+    from azure.mgmt.core.tools import is_valid_resource_id
     from ._validators import validate_delete_options
     is_lv_size = re.search('_L[0-9]+s', size, re.I)
     # we should return a dictionary with info like below
@@ -375,26 +371,34 @@ def update_disk_caching(model, caching_settings):
             _update(model, lun, value)
 
 
+def update_disk_caching_by_aaz(model, caching_settings):
+    def _update(model, lun, value):
+        luns = model.keys() if lun is None else [lun]
+        for lun_item in luns:
+            if lun_item not in model:
+                raise CLIError("Data disk with lun of '{}' doesn't exist. Existing luns: {}."
+                               .format(lun_item, list(model.keys())))
+            model[lun_item]['caching'] = value
+
+    if len(caching_settings) == 1 and '=' not in caching_settings[0]:
+        _update(model, None, caching_settings[0])
+    else:
+        for x in caching_settings:
+            if '=' not in x:
+                raise CLIError("usage error: please use 'LUN=VALUE' to configure caching on individual disk")
+            lun, value = x.split('=', 1)
+            lun = lun.lower()
+            lun = int(lun) if lun != 'os' else lun
+            _update(model, lun, value)
+
+
 def update_write_accelerator_settings(model, write_accelerator_settings):
     def _update(model, lun, value):
-        if isinstance(model, dict):
-            luns = model.keys() if lun is None else [lun]
-            for lun_item in luns:
-                if lun_item not in model:
-                    raise CLIError("data disk with lun of '{}' doesn't exist".format(lun_item))
-                model[lun_item]['writeAcceleratorEnabled'] = value
-        else:
-            if lun is None:
-                disks = [model.os_disk] + (model.data_disks or [])
-            elif lun == 'os':
-                disks = [model.os_disk]
-            else:
-                disk = next((d for d in model.data_disks if d.lun == lun), None)
-                if not disk:
-                    raise CLIError("data disk with lun of '{}' doesn't exist".format(lun))
-                disks = [disk]
-            for disk in disks:
-                disk.write_accelerator_enabled = value
+        luns = model.keys() if lun is None else [lun]
+        for lun_item in luns:
+            if lun_item not in model:
+                raise CLIError("data disk with lun of '{}' doesn't exist".format(lun_item))
+            model[lun_item]['write_accelerator_enabled'] = value
 
     if len(write_accelerator_settings) == 1 and '=' not in write_accelerator_settings[0]:
         _update(model, None, write_accelerator_settings[0].lower() == 'true')
@@ -656,23 +660,20 @@ def validate_vm_disk_trusted_launch(namespace, disk_security_profile):
         return
 
     security_type = disk_security_profile.security_type if hasattr(disk_security_profile, 'security_type') else None
-    if security_type.lower() == 'trustedlaunch':
+    if security_type and security_type.lower() == 'trustedlaunch':
         if namespace.enable_secure_boot is None:
             namespace.enable_secure_boot = True
         if namespace.enable_vtpm is None:
             namespace.enable_vtpm = True
         namespace.security_type = 'TrustedLaunch'
-    elif security_type.lower() == 'standard':
+    elif security_type and security_type.lower() == 'standard':
         logger.warning(UPGRADE_SECURITY_HINT)
 
 
 def validate_image_trusted_launch(namespace):
     from ._constants import UPGRADE_SECURITY_HINT
 
-    # set securityType to Standard by default if no inputs by end user
-    if namespace.security_type is None:
-        namespace.security_type = 'Standard'
-    if namespace.security_type.lower() != 'trustedlaunch':
+    if namespace.security_type and namespace.security_type.lower() != 'trustedlaunch':
         logger.warning(UPGRADE_SECURITY_HINT)
 
 
@@ -711,34 +712,48 @@ def import_aaz_by_profile(profile, module_name):
 
 
 def generate_ssh_keys_ed25519(private_key_filepath, public_key_filepath):
+    def _open(filename, mode):
+        return os.open(filename, flags=os.O_WRONLY | os.O_TRUNC | os.O_CREAT, mode=mode)
+
     from cryptography.hazmat.primitives import serialization
     from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PrivateKey
 
     ssh_dir = os.path.dirname(private_key_filepath)
     if not os.path.exists(ssh_dir):
-        os.makedirs(ssh_dir)
-        os.chmod(ssh_dir, 0o700)
+        os.makedirs(ssh_dir, mode=0o700)
 
-    private_key = Ed25519PrivateKey.generate()
-    public_key = private_key.public_key()
-    private_bytes = private_key.private_bytes(
-        encoding=serialization.Encoding.PEM,
-        format=serialization.PrivateFormat.OpenSSH,
-        encryption_algorithm=serialization.NoEncryption()
-    )
+    if os.path.isfile(private_key_filepath):
+        # Try to use existing private key if it exists.
+        with open(private_key_filepath, "rb") as f:
+            private_bytes = f.read()
+        private_key = serialization.load_ssh_private_key(private_bytes, password=None)
+        logger.warning("Private SSH key file '%s' was found in the directory: '%s'. "
+                       "A paired public key file '%s' will be generated.",
+                       private_key_filepath, ssh_dir, public_key_filepath)
 
-    with os.fdopen(os.open(private_key_filepath, flags=os.O_WRONLY | os.O_TRUNC | os.O_CREAT, mode=384, ), "w", ) as f:
-        f.write(
-            private_bytes.decode()
+    else:
+        # Otherwise generate new private key.
+        private_key = Ed25519PrivateKey.generate()
+
+        # The private key will look like:
+        # -----BEGIN OPENSSH PRIVATE KEY-----
+        # ...
+        # -----END OPENSSH PRIVATE KEY-----
+        private_bytes = private_key.private_bytes(
+            encoding=serialization.Encoding.PEM,
+            format=serialization.PrivateFormat.OpenSSH,
+            encryption_algorithm=serialization.NoEncryption()
         )
-    os.chmod(private_key_filepath, 0o600)
 
-    with open(public_key_filepath, 'w') as public_key_file:
-        s = public_key.public_bytes(
-            encoding=serialization.Encoding.OpenSSH,
-            format=serialization.PublicFormat.OpenSSH)
-        public_key = s.decode(encoding="utf8").replace("\n", "")
-        public_key_file.write(public_key)
-    os.chmod(public_key_filepath, 0o644)
+        with os.fdopen(_open(private_key_filepath, 0o600), "wb") as f:
+            f.write(private_bytes)
 
-    return public_key
+    public_key = private_key.public_key()
+    public_bytes = public_key.public_bytes(
+        encoding=serialization.Encoding.OpenSSH,
+        format=serialization.PublicFormat.OpenSSH)
+
+    with os.fdopen(_open(public_key_filepath, 0o644), 'wb') as f:
+        f.write(public_bytes)
+
+    return public_bytes.decode()
